@@ -3,9 +3,12 @@ Define an interface for creating an object, but let subclasses decide
 which class to instantiate. Factory Method lets a class defer
 instantiation to subclasses.
 """
+import os
 import re
 import pymongo
-from processor.helper.json.json_utils import get_field_value
+from processor.helper.json.json_utils import get_field_value, json_from_file, save_json_to_file
+from processor.helper.config.config_utils import get_test_json_dir, parsebool, config_value
+from processor.helper.file.file_utils import exists_file, exists_dir
 from processor.database.database import COLLECTION, get_documents
 from processor.comparison.comparison_functions import equality,\
     less_than, less_than_equal, greater_than, greater_than_equal, exists
@@ -14,7 +17,7 @@ from antlr4 import CommonTokenStream
 from processor.comparison.comparisonantlr.comparatorLexer import comparatorLexer
 from processor.comparison.comparisonantlr.comparatorParser import comparatorParser
 from processor.comparison.comparisonantlr.rule_interpreter import RuleInterpreter
-
+from processor.helper.config.rundata_utils import get_dbtests
 from processor.logging.log_handler import getlogger
 
 
@@ -111,6 +114,34 @@ def interpret_additional_operations(roperand):
     return value
 
 
+def opa_binary():
+    opa_exe = None
+    opa_enabled = config_value("OPA", "opa")
+    if opa_enabled:
+        opa_exe = config_value("OPA", "opaexe")
+        if opa_exe and exists_file(opa_exe):
+            pass
+        else:
+            opa_exe = None
+    return opa_exe
+
+
+def get_rego_rule_filename(rego_file, container):
+    rego_file_name = None
+    json_dir = get_test_json_dir()
+    if exists_dir(json_dir):
+        rego_file_name = '%s/%s/%s' % (json_dir, container, rego_file)
+        if exists_file(rego_file_name):
+            pass
+        else:
+            rego_file_name = None
+    return rego_file_name
+
+
+def compare(inputjson, rulestr):
+   pass
+
+
 class Comparator:
     """
     Declare the factory method, which returns a comparator object.
@@ -143,22 +174,128 @@ class ComparatorV01:
         self.container = container
         self.dbname = dbname
         self.collection_data = collection_data
+        self.testcase = testcase
         loperand = get_field_value(testcase, 'attribute')
         value = get_field_value(testcase, 'comparison')
         rule = get_field_value(testcase, 'rule')
-        if loperand and value:
+        isrego = get_field_value(testcase, 'type')
+        self.snapshots = []
+        if isrego and isrego == 'rego':
+            self.format = TESTCASEV2
+            self.rule = rule
+            self.type = 'rego'
+        elif loperand and value:
             self.format = TESTCASEV1
             self.snapshot_id = get_field_value(testcase, 'snapshotId')
             coll_val = get_field_value(self.collection_data, self.snapshot_id)
             self.collection = coll_val if coll_val else COLLECTION
             self.loperand = loperand
             self.is_not, self.op, self.roperand, self.extras = get_operator_roperand(value)
+            self.type = 'prancer'
         elif rule:
             self.format = TESTCASEV2
             self.rule = rule
+            self.type = 'prancer'
         else:
             self.format = None
 
+    def process_rego_test_case(self):
+        result = False
+        opa_exe = opa_binary()
+        if not opa_exe:
+            return result
+        rule_expr = get_field_value(self.testcase, 'eval')
+        if not rule_expr:
+            rule_expr = 'data.rule.rulepass'
+        sid = self.testcase['snapshotId'][0]
+        inputjson = self.get_snaphotid_doc(sid)
+        if inputjson:
+            save_json_to_file(inputjson, '/tmp/input.json')
+            rego_rule = self.rule
+            rego_match=re.match(r'^file\((.*)\)$', rego_rule, re.I)
+            if rego_match:
+                rego_file = get_rego_rule_filename(rego_match.groups()[0], self.container)
+                if rego_file:
+                    pass
+                else:
+                    rego_file = None
+            else:
+                rego_txt = [
+                    "package rule",
+                    "default rulepass = false",
+                    "rulepass = true{",
+                    "   %s" % rego_rule,
+                    "}", ""
+                ]
+                rego_file = '/tmp/input.rego'
+                open(rego_file, 'w').write('\n'.join(rego_txt))
+            if rego_file:
+                os.system('%s eval -i /tmp/input.json -d %s "%s" > /tmp/a.json' % (opa_exe, rego_file, rule_expr))
+                resultval = json_from_file('/tmp/a.json')
+                if resultval:
+                    resultbool = resultval['result'][0]['expressions'][0]['value'] # [get_field_value(resultval, 'result[0].expressions[0].value')
+                    if resultbool:
+                        result = parsebool(resultbool)
+            else:
+                result = False
+        return result
+
+
+
+    def get_snaphotid_doc_old(self, sid, container):
+        doc = None
+        json_dir = get_test_json_dir()
+        if exists_dir(json_dir):
+            fname = '%s/%s/snapshots/%s' % (json_dir, container, sid)
+            if exists_file(fname):
+                json_data = json_from_file(fname)
+                if json_data and 'json' in json_data:
+                    doc = json_data['json']
+                    self.snapshots.append({
+                        'id': json_data['snapshotId'],
+                        'path': json_data['path'],
+                        'structure': json_data['structure'],
+                        'reference': json_data['reference'],
+                        'source': json_data['source']
+                    })
+        return doc
+
+
+    def get_snaphotid_doc(self, sid):
+        doc = None
+        isdb_fetch = get_dbtests()
+        if isdb_fetch:
+            dbname = self.dbname
+            coll = self.collection_data[sid] if sid in self.collection_data else COLLECTION
+            docs = get_documents(coll, {'snapshotId': sid}, dbname,
+                                 sort=[('timestamp', pymongo.DESCENDING)], limit=1)
+            logger.debug('Number of Snapshot Documents: %s', len(docs))
+            if docs and len(docs):
+                doc = docs[0]['json']
+                self.snapshots.append({
+                    'id': docs[0]['snapshotId'],
+                    'path': docs[0]['path'],
+                    'structure': docs[0]['structure'],
+                    'reference': docs[0]['reference'],
+                    'source': docs[0]['source']
+                })
+        else:
+            json_dir = '%s%s' % (get_test_json_dir(), self.container)
+            if exists_dir(json_dir):
+                fname = '%s/snapshots/%s' % (json_dir, sid)
+                if exists_file(fname):
+                    json_data = json_from_file(fname)
+                    if json_data and 'json' in json_data:
+                        doc = json_data['json']
+                        self.snapshots.append({
+                            'id': json_data['snapshotId'],
+                            'path': json_data['path'],
+                            'structure': json_data['structure'],
+                            'reference': json_data['reference'],
+                            'source': json_data['source']
+                        })
+        return doc
+
     def validate(self):
         result_val = {"result": "failed"}
         if self.format == TESTCASEV1:
@@ -167,19 +304,6 @@ class ComparatorV01:
                                      sort=[('timestamp', pymongo.DESCENDING)],
                                      query={'snapshotId': self.snapshot_id},
                                      limit=1)
-                # docs = [{
-                # "_id": "5c24af787456217c485ad1e6",
-                # "checksum": "7d814f2f82a32ea91ef37de9e11d0486",
-                # "collection": "microsoftcompute",
-                # "json":{
-                #     "id": 124,
-                #     "location": "eastus2",
-                #     "name": "mno-nonprod-shared-cet-eastus2-tab-as03"
-                # },
-                # "queryuser": "ajeybk1@kbajeygmail.onmicrosoft.com",
-                # "snapshotId": 1,
-                # "timestamp": 1545908086831
-                # }]
                 logger.info('Number of Snapshot Documents: %s', len(docs))
                 if docs and len(docs):
                     self.data = docs[0]['json']
@@ -194,8 +318,6 @@ class ComparatorV01:
                             'reference': docs[0]['reference'],
                             'source': docs[0]['source']
                         }]
-                    # else:
-                    #     result_val['reason'] = 'Unsupported comparison operator: %s' % self.op
                 else:
                     result_val.update({
                         "result": "skipped",
@@ -207,23 +329,28 @@ class ComparatorV01:
                     "reason": "Missing snapshotId for testcase"
                 })
         elif self.format == TESTCASEV2:
-            logger.info('#' * 75)
-            logger.info('Actual Rule: %s', self.rule)
-            input_stream = InputStream(self.rule)
-            lexer = comparatorLexer(input_stream)
-            stream = CommonTokenStream(lexer)
-            parser = comparatorParser(stream)
-            tree = parser.expression()
-            children = []
-            for child in tree.getChildren():
-                children.append((child.getText()))
-            logger.info('*' * 50)
-            logger.debug("All the parsed tokens: %s", children)
-            otherdata = {'dbname': self.dbname, 'snapshots': self.collection_data, 'container': self.container}
-            r_i = RuleInterpreter(children, **otherdata)
-            result = r_i.compare()
-            result_val["result"] = "passed" if result else "failed"
-            result_val['snapshots'] = r_i.get_snapshots()
+            if self.type == 'rego':
+                result = self.process_rego_test_case()
+                result_val["result"] = "passed" if result else "failed"
+                result_val['snapshots'] = self.snapshots
+            else:
+                logger.info('#' * 75)
+                logger.info('Actual Rule: %s', self.rule)
+                input_stream = InputStream(self.rule)
+                lexer = comparatorLexer(input_stream)
+                stream = CommonTokenStream(lexer)
+                parser = comparatorParser(stream)
+                tree = parser.expression()
+                children = []
+                for child in tree.getChildren():
+                    children.append((child.getText()))
+                logger.info('*' * 50)
+                logger.debug("All the parsed tokens: %s", children)
+                otherdata = {'dbname': self.dbname, 'snapshots': self.collection_data, 'container': self.container}
+                r_i = RuleInterpreter(children, **otherdata)
+                result = r_i.compare()
+                result_val["result"] = "passed" if result else "failed"
+                result_val['snapshots'] = r_i.get_snapshots()
         else:
             result_val.update({
                 "result": "skipped",
@@ -243,12 +370,21 @@ class ComparatorV02(ComparatorV01):
         return ComparatorV01.validate(self)
 
 
-# if __name__ == "__main__":
-#     comparator = Comparator('0.1', 'validator', {}, {
-#         "testId": "4",
-#         "snapshotId1": "1",
-#         "attribute": "id",
-#         "comparison": "gt0 10"
-#     })
-#     val = comparator.validate()
-#     print(val)
\ No newline at end of file
+def main(container):
+    from processor.connector.validation import run_container_validation_tests_filesystem
+    try:
+        run_container_validation_tests_filesystem(container)
+        return True
+    except Exception as ex:
+        print("Exception: %s" % ex)
+        return False
+
+
+if __name__ == '__main__':
+    import sys
+    if len(sys.argv) > 1:
+        main(sys.argv[1])
+    else:        
+        comparator = Comparator('0.1', 'validator', {}, {"testId": "4", "snapshotId1": "1", "attribute": "id", "comparison": "gt0 10"})
+        val = comparator.validate()
+        print(val)
